package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.dao.DispositivoEmbarcadaDAO;
import com.rjconsultores.ventaboletos.entidad.CtrlSerieEmbarcada;
import com.rjconsultores.ventaboletos.entidad.DispositivoEmbarcada;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.Estado;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.relatorios.impl.RelatorioAutorizacaoUsoSerieEmbarcada;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.service.EstadoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEstandar;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxPuntoVenta;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderAutorizacaoUsoSerieEmbarcada;
import com.trg.search.Search;

@Controller("busquedaAutorizacaoUsoSerieEmbarcadaController")
@Scope("prototype")
public class BusquedaAutorizacaoUsoSerieEmbarcadaController extends MyGenericForwardComposer {
	private static final long serialVersionUID = 5333221264682600531L;
	
	@Autowired
	private transient PagedListWrapper<CtrlSerieEmbarcada> plwAutorizacaoUsoSerieEmbarcada;
	
	@Autowired
	private EstadoService estadoService;
	
	@Autowired
	private DispositivoEmbarcadaDAO dispositivoEmbarcadaDAO;  
	
	@Autowired
	private DataSource dataSourceRead;

	private MyComboboxEstandar cmbEmpresa;
	private List<Empresa> lsEmpresa;

	private MyComboboxEstandar cmbDescricao;
	private List<DispositivoEmbarcada> lsDescricao;
	
	private Combobox cmbEstado;
	private List<Estado> lsEstado;

	private Textbox txtSerie;
	private Textbox txtMei;
	
	private MyComboboxPuntoVenta cmbPuntoVenta;

	private Paging pagingAutorizacaoUsoSerieEmbarcada;
	private MyListbox autorizacaoUsoSerieEmbarcadaList;
	private Search autorizacaoUsoSerieEmbarcadaBusqueda;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);
		
		this.lsEmpresa = UsuarioLogado.getUsuarioLogado().getEmpresa();
		this.lsDescricao = dispositivoEmbarcadaDAO.buscarTodos();
		this.lsEstado = estadoService.obtenerTodos();
		
		Empresa todasEmpresas = new Empresa();
		todasEmpresas.setEmpresaId(-1);
		todasEmpresas.setNombempresa("TODAS");
		
		this.lsEmpresa.add(todasEmpresas);
		
		Estado todosEstados = new Estado();
		todosEstados.setEstadoId(-1);
		todosEstados.setCveestado("TODOS");
		todosEstados.setNombestado("TODOS");
		
		this.lsEstado.add(todosEstados);
		
		DispositivoEmbarcada todasDescricoes = new DispositivoEmbarcada();
		todasDescricoes.setDispositivoEmbarcadaId(-1l);
		todasDescricoes.setDescricao("TODAS");
		
		this.lsDescricao.add(todasDescricoes);
		
		autorizacaoUsoSerieEmbarcadaList.setItemRenderer(new RenderAutorizacaoUsoSerieEmbarcada());

		autorizacaoUsoSerieEmbarcadaList.addEventListener("onDoubleClick", new EventListener() {
			@Override
			public void onEvent(Event event) throws Exception {
				CtrlSerieEmbarcada a = (CtrlSerieEmbarcada) autorizacaoUsoSerieEmbarcadaList.getSelected();
				editar(a);
			}
		});

		refreshLista();

		cmbEmpresa.focus();
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void editar(CtrlSerieEmbarcada a) {
		if (a == null) {
			return;
		}

		Map args = new HashMap();
		args.put("autorizacaoUsoSerieEmbarcada", a);
		args.put("autorizacaoUsoSerieEmbarcadaList", autorizacaoUsoSerieEmbarcadaList);

		openWindow("/gui/catalogos/editarAutorizacaoUsoSerieEmbarcada.zul",
				Labels.getLabel("editarAutorizacaoUsoSerieEmbarcadaController.window.title"), args, MODAL);
	}

	private void refreshLista() {
		HibernateSearchObject<CtrlSerieEmbarcada> autorizacaoUsoSerieEmbarcadaBusqueda = 
				new HibernateSearchObject<CtrlSerieEmbarcada>(CtrlSerieEmbarcada.class, pagingAutorizacaoUsoSerieEmbarcada.getPageSize());

		if (txtMei.getText() != null && !txtMei.getText().isEmpty()) {
			autorizacaoUsoSerieEmbarcadaBusqueda.addFilterLike("dispositivoEmbarcada.imei", "%" + txtMei.getText().trim().concat("%"));
		}

		if (txtSerie.getText() != null && !txtSerie.getText().isEmpty()) {
			autorizacaoUsoSerieEmbarcadaBusqueda.addFilterLike("serie", "%" + txtSerie.getText().trim().concat("%"));
		}
		
		if (cmbEmpresa.getSelectedItem() != null) {
			Empresa e = (Empresa) cmbEmpresa.getSelectedItem().getValue();
			
			if (e.getEmpresaId().intValue() != -1) {
				autorizacaoUsoSerieEmbarcadaBusqueda.addFilterEqual("empresa.empresaId", e.getEmpresaId());
			}
		}
		
		if (cmbPuntoVenta.getSelectedItem() != null) {
			PuntoVenta p = (PuntoVenta) cmbPuntoVenta.getSelectedItem().getValue();
			
			if (p.getPuntoventaId().intValue() != -1) {
				autorizacaoUsoSerieEmbarcadaBusqueda.addFilterEqual("puntoventa.puntoventaId", p.getPuntoventaId());
			}
		}
		
		if (cmbDescricao.getSelectedItem() != null) {
			DispositivoEmbarcada d = (DispositivoEmbarcada) cmbDescricao.getSelectedItem().getValue();
			
			if (d.getDispositivoEmbarcadaId().intValue() != -1) {
				autorizacaoUsoSerieEmbarcadaBusqueda.addFilterEqual("dispositivoEmbarcada.dispositivoEmbarcadaId", d.getDispositivoEmbarcadaId());
			}
		}
		
		if (cmbEstado.getSelectedItem() != null) {
			Estado e = (Estado) cmbEstado.getSelectedItem().getValue();
			
			if (e.getEstadoId().intValue() != -1) {
				autorizacaoUsoSerieEmbarcadaBusqueda.addFilterEqual("estado.estadoId", e.getEstadoId());
			}
		}

		autorizacaoUsoSerieEmbarcadaBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);
		autorizacaoUsoSerieEmbarcadaBusqueda.addSortDesc("ctrlSerieEmbarcadaId");

		plwAutorizacaoUsoSerieEmbarcada.init(autorizacaoUsoSerieEmbarcadaBusqueda, autorizacaoUsoSerieEmbarcadaList, pagingAutorizacaoUsoSerieEmbarcada);

		if (autorizacaoUsoSerieEmbarcadaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("autorizacaoUsoSerieEmbarcadaController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	public void onClick$btnPesquisa(Event ev) {
		refreshLista();
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista();
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public void onClick$btnExportar(Event ev) throws SQLException, Exception {
		List<CtrlSerieEmbarcada> selecao = autorizacaoUsoSerieEmbarcadaList.getListData();
		if (selecao == null || selecao.isEmpty()) {
			return;
		}
		
		HashMap<String, Object> parametros = new HashMap<String, Object>();
		parametros.put("NOME_RELATORIO", Labels.getLabel("autorizacaoUsoSerieEmbarcadaController.relatorio.titulo"));
		parametros.put("USUARIO", UsuarioLogado.getUsuarioLogado().getUsuarioId().toString());
		parametros.put("USUARIO_NOME", UsuarioLogado.getUsuarioLogado().getNombusuario());
		
		Relatorio relatorio = new RelatorioAutorizacaoUsoSerieEmbarcada(selecao, dataSourceRead.getConnection(), parametros);
		
		Map args = new HashMap();
		args.put("relatorio", relatorio);

		openWindow("/component/reportView.zul",
				Labels.getLabel("autorizacaoUsoSerieEmbarcadaController.relatorio.titulo"), args, MODAL);
	}
	

	public void onClick$btnNovo(Event ev) {
		editar(new CtrlSerieEmbarcada());
	}

	public Combobox getCmbEmpresa() {
		return cmbEmpresa;
	}

	public void setCmbEmpresa(MyComboboxEstandar cmbEmpresa) {
		this.cmbEmpresa = cmbEmpresa;
	}

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

	public Textbox getTxtMei() {
		return txtMei;
	}

	public void setTxtMei(Textbox txtMei) {
		this.txtMei = txtMei;
	}

	public MyComboboxEstandar getCmbDescricao() {
		return cmbDescricao;
	}

	public void setCmbDescricao(MyComboboxEstandar cmbDescricao) {
		this.cmbDescricao = cmbDescricao;
	}

	public List<DispositivoEmbarcada> getLsDescricao() {
		return lsDescricao;
	}

	public void setLsDescricao(List<DispositivoEmbarcada> lsDescricao) {
		this.lsDescricao = lsDescricao;
	}

	public Combobox getCmbEstado() {
		return cmbEstado;
	}

	public void setCmbEstado(Combobox cmbEstado) {
		this.cmbEstado = cmbEstado;
	}

	public List<Estado> getLsEstado() {
		return lsEstado;
	}

	public void setLsEstado(List<Estado> lsEstado) {
		this.lsEstado = lsEstado;
	}

	public MyComboboxPuntoVenta getCmbPuntoVenta() {
		return cmbPuntoVenta;
	}

	public void setCmbPuntoVenta(MyComboboxPuntoVenta cmbPuntoVenta) {
		this.cmbPuntoVenta = cmbPuntoVenta;
	}

	public Textbox getTxtSerie() {
		return txtSerie;
	}

	public void setTxtSerie(Textbox txtSerie) {
		this.txtSerie = txtSerie;
	}

	public PagedListWrapper<CtrlSerieEmbarcada> getPlwAutorizacaoUsoSerieEmbarcada() {
		return plwAutorizacaoUsoSerieEmbarcada;
	}

	public void setPlwAutorizacaoUsoSerieEmbarcada(PagedListWrapper<CtrlSerieEmbarcada> plwAutorizacaoUsoSerieEmbarcada) {
		this.plwAutorizacaoUsoSerieEmbarcada = plwAutorizacaoUsoSerieEmbarcada;
	}

	public Paging getPagingAutorizacaoUsoSerieEmbarcada() {
		return pagingAutorizacaoUsoSerieEmbarcada;
	}

	public void setPagingAutorizacaoUsoSerieEmbarcada(Paging pagingAutorizacaoUsoSerieEmbarcada) {
		this.pagingAutorizacaoUsoSerieEmbarcada = pagingAutorizacaoUsoSerieEmbarcada;
	}

	public MyListbox getAutorizacaoUsoSerieEmbarcadaList() {
		return autorizacaoUsoSerieEmbarcadaList;
	}

	public void setAutorizacaoUsoSerieEmbarcadaList(MyListbox autorizacaoUsoSerieEmbarcadaList) {
		this.autorizacaoUsoSerieEmbarcadaList = autorizacaoUsoSerieEmbarcadaList;
	}

	public Search getAutorizacaoUsoSerieEmbarcadaBusqueda() {
		return autorizacaoUsoSerieEmbarcadaBusqueda;
	}

	public void setAutorizacaoUsoSerieEmbarcadaBusqueda(Search autorizacaoUsoSerieEmbarcadaBusqueda) {
		this.autorizacaoUsoSerieEmbarcadaBusqueda = autorizacaoUsoSerieEmbarcadaBusqueda;
	}
}
