package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.Executions;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.util.Clients;
import org.zkoss.zul.Messagebox;
import org.zkoss.zul.api.Button;

import com.rjconsultores.ventaboletos.entidad.EmbarqueCatracaRodoviaria;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.Parada;
import com.rjconsultores.ventaboletos.service.EmbarqueCatracaRodoviariaService;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.utilerias.ApplicationProperties;
import com.rjconsultores.ventaboletos.utilerias.CustomEnum;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEmpresa;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxParada;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;

@Controller("editarConfiguracaoCodBarrasController")
@Scope("prototype")
public class EditarConfiguracaoCodBarrasController extends MyGenericForwardComposer {
	private static final long serialVersionUID = 1L;

	@Autowired
	private EmpresaService empresaService;

	@Autowired
	private EmbarqueCatracaRodoviariaService embarqueCatracaService;

	private MyListbox codBarrasList;
	private EmbarqueCatracaRodoviaria embarqueCatracaRodoviaria;
	private List<Empresa> lsEmpresa;
	private List<Parada> lsParada;

	private MyComboboxEmpresa cmbEmpresa;
	private MyComboboxParada cmbParada;
	private MyTextbox txtEmpresaRodID;
	private MyTextbox txtSequenciaRod;
	private Button btnApagar;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);
		lsEmpresa = empresaService.obtenerTodos();

		codBarrasList = (MyListbox) Executions.getCurrent().getArg().get("codBarrasList");
		embarqueCatracaRodoviaria = (EmbarqueCatracaRodoviaria) Executions.getCurrent().getArg().get("embarqueCatraca");
		if (embarqueCatracaRodoviaria != null) {
			txtEmpresaRodID.setText(embarqueCatracaRodoviaria.getEmpresaRod().toString());
			txtSequenciaRod.setText(embarqueCatracaRodoviaria.getSequenciaRod().toString());
			cmbEmpresa.setText(embarqueCatracaRodoviaria.getEmpresa().getNombempresa());
			cmbParada.setText(embarqueCatracaRodoviaria.getParada().getDescparada());
			cmbParada.setComboItemByParada(embarqueCatracaRodoviaria.getParada());
		} else {
			btnApagar.setVisible(false);
			embarqueCatracaRodoviaria = new EmbarqueCatracaRodoviaria();
		}
		
		if(ApplicationProperties.getInstance().isCustomHabilitado(CustomEnum.IS_TAXA_EMBARQUE_IMPRESSA_QRCODE.getDescricao())) {
			txtEmpresaRodID.setText("99999");
			txtSequenciaRod.setText("99999");
			
			txtEmpresaRodID.setDisabled(true);
			txtSequenciaRod.setDisabled(true);
		}
	}

	public void onClick$btnSalvar(Event event) {
		
		if(!validarTamanhoDigitosCampos()) {
			return;
		}
		embarqueCatracaRodoviaria.setActivo(true);
		embarqueCatracaRodoviaria.setEmpresaRod(new Integer(txtEmpresaRodID.getText()));
		embarqueCatracaRodoviaria.setSequenciaRod(new Integer(txtSequenciaRod.getText()));
		embarqueCatracaRodoviaria.setEmpresa((Empresa) cmbEmpresa.getSelectedItem().getValue());
		embarqueCatracaRodoviaria.setParada((Parada) cmbParada.getSelectedItem().getValue());

		codBarrasList.removeItem(embarqueCatracaRodoviaria);
		codBarrasList.addItemNovo(embarqueCatracaService.suscribir(embarqueCatracaRodoviaria));
		closeWindow();
	}

	private boolean validarTamanhoDigitosCampos() {
		
		boolean valido = true;
		
		if(txtEmpresaRodID.getText().length() > 4) {
            Clients.wrongValue(txtEmpresaRodID, Labels.getLabel("editarConfiguracaoCodBarrasController.MSG.empresaRodQtdDigitos"));
            valido = false;
		}
		if(txtSequenciaRod.getText().length() > 8) {
            Clients.wrongValue(txtSequenciaRod, Labels.getLabel("editarConfiguracaoCodBarrasController.MSG.sequenciaRodQtdDigitos"));
            valido = false;
		}
		return valido;
		
	}
	public void onClick$btnApagar(Event event) throws InterruptedException {
		int resp = Messagebox.show(
				Labels.getLabel("editarConfiguracaoCodBarrasController.MSG.borrarPergunta"),
				Labels.getLabel("editarConfiguracaoCodBarrasController.title"),
				Messagebox.YES | Messagebox.NO, Messagebox.QUESTION);

		if (resp == Messagebox.YES) {

			embarqueCatracaService.borrar(embarqueCatracaRodoviaria);

			Messagebox.show(
					Labels.getLabel("editarConfiguracaoCodBarrasController.MSG.borrarOK"),
					Labels.getLabel("editarConfiguracaoCodBarrasController.title"),
					Messagebox.OK, Messagebox.INFORMATION);

			codBarrasList.removeItem(embarqueCatracaRodoviaria);

			closeWindow();
		}
	}

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

	public List<Parada> getLsParada() {
		return lsParada;
	}

	public void setLsParada(List<Parada> lsParada) {
		this.lsParada = lsParada;
	}

}
