package com.rjconsultores.ventaboletos.web.gui.controladores.expressos;

import static org.junit.Assert.assertEquals;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Base64;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.media.AMedia;
import org.zkoss.util.media.Media;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Filedownload;
import org.zkoss.zhtml.Fileupload;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.Executions;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zk.ui.event.UploadEvent;
import org.zkoss.zul.Checkbox;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Messagebox;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.SolicitudExpreso;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.service.LogAuditoriaService;
import com.rjconsultores.ventaboletos.service.SolicitudExpresosService;
import com.rjconsultores.ventaboletos.web.utilerias.MyDatebox;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderCargaContratoExpresos;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderExpresosPorCotizar;

import javax.sql.rowset.serial.SerialBlob;
import javax.swing.JFileChooser;

import oracle.sql.BLOB;

@Controller("cargaContratoExpressosController")
@Scope("prototype")
public class CargaContratoExpressosController extends MyGenericForwardComposer{
	private static final long serialVersionUID = 1L;

	@Autowired
    private transient PagedListWrapper<SolicitudExpreso> plwTrayectosExpresos;
	
	@Autowired
	private EmpresaService empresaService;
	
	@Autowired
	SolicitudExpresosService solicitudExpresosService;
	
	@Autowired
    private LogAuditoriaService logAuditoriaService;
	
	private List<Empresa> lsEmpresa;
	private Combobox cmbEmpresa;
	private MyListbox expresosList;
	private Paging pagingExpresos;
	private Checkbox ckServiciosInactivos;
	
	private MyDatebox txtFechaInicio;
	private MyDatebox txtFechaFin;
	
	SolicitudExpreso expreso;
	SolicitudExpreso expresoClone;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsEmpresa = empresaService.obtenerTodos();
		super.doAfterCompose(comp);
		
		expresosList.setItemRenderer(new RenderCargaContratoExpresos(this, new EventListener() {

            @Override
            public void onEvent(Event event) throws Exception {
            	expreso = (SolicitudExpreso) event.getTarget().getAttribute("data");
            	expreso.clonar();
            	expresoClone = expreso.getCloneObject();
            	
            	Media contrato = Fileupload.get();
            	
            	if(contrato != null) {
            		if(contrato.getFormat().equalsIgnoreCase("pdf")) {
    				    InputStream inputStream = contrato.getStreamData();
    				    byte[] bytesIs = IOUtils.toByteArray(inputStream);
    					
    				    expreso.setDocContrato(bytesIs);
    			        
    				    solicitudExpresosService.actualizacion(expreso);
    				    
    				    logAuditoriaService.auditar(expresoClone, expreso, null);
    				    
    				    Messagebox.show(
    							Labels.getLabel("expresoController.MSG.archivoGuardadoConExito"),
    							Labels.getLabel("winExpressoCargaContrato.title"),
    							Messagebox.OK, Messagebox.INFORMATION);
    				} else {
    					Messagebox.show(
    							Labels.getLabel("cargaContratoController.MSG.errorFormatoContrato") + " " + contrato,
    							Labels.getLabel("winExpressoCargaContrato.title"),
    							Messagebox.OK, Messagebox.ERROR);
    				}
            	}else {
            		Messagebox.show(
							Labels.getLabel("expresoController.MSG.archivoNoSeleccionado"),
							Labels.getLabel("winExpressoCargaContrato.title"),
							Messagebox.OK, Messagebox.ERROR);
            	}
            }
        }));
		
		refreshLista();
	}
	
	private void refreshLista() {
		HibernateSearchObject<SolicitudExpreso> buscarExpresos =
                new HibernateSearchObject<SolicitudExpreso>(SolicitudExpreso.class, pagingExpresos.getPageSize());
		
		Date fechaInicio = txtFechaInicio.getValue();
		if(fechaInicio != null) {
			buscarExpresos.addFilterGreaterOrEqual("fechaSolicitud", fechaInicio);
		}
		
		Date fechaFin = txtFechaFin.getValue();
		if(fechaFin != null) {
			buscarExpresos.addFilterLessOrEqual("fechaSolicitud", fechaFin);
		}
		
		if(ckServiciosInactivos.isChecked()) {
			buscarExpresos.addFilterEqual("ACTIVO", true);
		}
		
		List<Integer> filterStatusSolicitudExpreso = new ArrayList<Integer>();
		filterStatusSolicitudExpreso.add(1);
		filterStatusSolicitudExpreso.add(2);
		
		buscarExpresos.addFilterIn("statusSolicitudExpresoId", filterStatusSolicitudExpreso);
		
		plwTrayectosExpresos.init(buscarExpresos, expresosList, pagingExpresos);
	}
	
	public void onUpload(UploadEvent event) throws IOException, InterruptedException, CloneNotSupportedException {
		if(expreso == null) {
			Messagebox.show(
					Labels.getLabel("cargaContratoController.MSG.errorExpresoNull"),
					Labels.getLabel("winExpressoCargaContrato.title"),
					Messagebox.OK, Messagebox.ERROR);
		}else {
			Media media = event.getMedia();
			
			if(media.getFormat().equalsIgnoreCase("pdf")) {
			    InputStream inputStream = media.getStreamData();
			    byte[] bytesIs = IOUtils.toByteArray(inputStream);
				
				expreso.setDocContrato(bytesIs);
		        
		        solicitudExpresosService.actualizacion(expreso);
		        
		        logAuditoriaService.auditar(expresoClone, expreso, null);
			} else {
				Messagebox.show(
						Labels.getLabel("cargaContratoController.MSG.errorFormatoContrato") + " " + media,
						Labels.getLabel("winExpressoCargaContrato.title"),
						Messagebox.OK, Messagebox.ERROR);
			}
		}
	}
	
	public void onClick$btnRefresh(Event ev) {
        refreshLista();
    }
	
	public void onClick$btnPesquisa(Event ev) {
        refreshLista();
    }
	
	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

	public Combobox getCmbEmpresa() {
		return cmbEmpresa;
	}

	public void setCmbEmpresa(Combobox cmbEmpresa) {
		this.cmbEmpresa = cmbEmpresa;
	}

	public MyListbox getExpresosList() {
		return expresosList;
	}

	public void setExpresosList(MyListbox expresosList) {
		this.expresosList = expresosList;
	}

	public Paging getPagingExpresos() {
		return pagingExpresos;
	}

	public void setPagingExpresos(Paging pagingExpresos) {
		this.pagingExpresos = pagingExpresos;
	}

	public MyDatebox getTxtFechaInicio() {
		return txtFechaInicio;
	}

	public void setTxtFechaInicio(MyDatebox txtFechaInicio) {
		this.txtFechaInicio = txtFechaInicio;
	}

	public PagedListWrapper<SolicitudExpreso> getPlwTrayectosExpresos() {
		return plwTrayectosExpresos;
	}

	public void setPlwTrayectosExpresos(PagedListWrapper<SolicitudExpreso> plwTrayectosExpresos) {
		this.plwTrayectosExpresos = plwTrayectosExpresos;
	}
}
