package com.rjconsultores.ventaboletos.web.gui.controladores.expressos;

import java.io.InputStream;
import java.util.Date;
import java.util.List;

import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.media.Media;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Fileupload;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Checkbox;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Messagebox;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.SolicitudExpreso;
import com.rjconsultores.ventaboletos.entidad.TrayectosExpresos;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.service.LogAuditoriaService;
import com.rjconsultores.ventaboletos.service.SolicitudExpresosService;
import com.rjconsultores.ventaboletos.web.utilerias.MyDatebox;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderCumplimientoServicioExpresos;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderDocumentosExpresos;

@Controller("cumplimientoServicioExpresosController")
@Scope("prototype")
public class CumplimientoServicioExpresosController extends MyGenericForwardComposer{
	private static final long serialVersionUID = 1L;
	
	@Autowired
    private transient PagedListWrapper<SolicitudExpreso> plwTrayectosExpresos;
	
	@Autowired
	private EmpresaService empresaService;
	
	@Autowired
	SolicitudExpresosService solicitudExpreso;
	
	@Autowired
    private LogAuditoriaService logAuditoriaService;
	
	private List<Empresa> lsEmpresa;
	private Paging pagingExpresos;
	private Combobox cmbEmpresa;
	private MyListbox expresosList;
	private Checkbox ckServiciosInactivos;
	private MyDatebox txtFechaInicio;
	private MyDatebox txtFechaFin;
	
	SolicitudExpreso expreso;
	SolicitudExpreso expresoClone;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsEmpresa = empresaService.obtenerTodos();
		super.doAfterCompose(comp);
		
		expresosList.setItemRenderer(new RenderCumplimientoServicioExpresos(new EventListener() {			
			@Override
			public void onEvent(Event arg0) throws Exception {
				expreso = (SolicitudExpreso)arg0.getTarget().getAttribute("data");
				expreso.clonar();
				expresoClone = expreso.getCloneObject();
				
				Media cumplimiento = Fileupload.get();
				
				if(cumplimiento.getFormat().equalsIgnoreCase("pdf")) {
					
					int resp = Messagebox.show(
							Labels.getLabel("cumplimientoServicioExpresoController.MSG.servicioCumplido"),
							Labels.getLabel("indexController.mniExpressosCumplimientoServicio.label"),
							Messagebox.YES | Messagebox.NO, Messagebox.QUESTION);

					if (resp == Messagebox.YES) {
						InputStream inputStream = cumplimiento.getStreamData();
					    byte[] bytesIs = IOUtils.toByteArray(inputStream);
						
					    expreso.setDocCumplimientoServicio(bytesIs);
				        
					    solicitudExpreso.actualizacion(expreso);
					    
					    logAuditoriaService.auditar(expresoClone, expreso, null);
					    
					    refreshLista();
					}
				} else {
					Messagebox.show(
							Labels.getLabel("cargaContratoController.MSG.errorFormatoContrato") + " " + cumplimiento,
							Labels.getLabel("winExpressoCargaContrato.title"),
							Messagebox.OK, Messagebox.ERROR);
				}
			}
		}));
		
		refreshLista();
	}
	
	private void refreshLista() {
		HibernateSearchObject<SolicitudExpreso> buscarExpresos =
                new HibernateSearchObject<SolicitudExpreso>(SolicitudExpreso.class, pagingExpresos.getPageSize());
		
		Date fechaInicio = txtFechaInicio.getValue();
		if(fechaInicio != null) {
			buscarExpresos.addFilterGreaterOrEqual("fechaSolicitud", fechaInicio);
		}
		
		Date fechaFin = txtFechaFin.getValue();
		if(fechaFin != null) {
			buscarExpresos.addFilterLessOrEqual("fechaSolicitud", fechaFin);
		}
		
		if(ckServiciosInactivos.isChecked()) {
			buscarExpresos.addFilterEqual("ACTIVO", true);
		}
		
		plwTrayectosExpresos.init(buscarExpresos, expresosList, pagingExpresos);
	}
	
	public void onClick$btnRefresh(Event ev) {
        refreshLista();
    }
	
	public void onClick$btnPesquisa(Event ev) {
        refreshLista();
    }
	
	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

	public Combobox getCmbEmpresa() {
		return cmbEmpresa;
	}

	public void setCmbEmpresa(Combobox cmbEmpresa) {
		this.cmbEmpresa = cmbEmpresa;
	}

	public MyListbox getExpresosList() {
		return expresosList;
	}

	public void setExpresosList(MyListbox expresosList) {
		this.expresosList = expresosList;
	}

	public Paging getPagingExpresos() {
		return pagingExpresos;
	}

	public void setPagingExpresos(Paging pagingExpresos) {
		this.pagingExpresos = pagingExpresos;
	}

	public PagedListWrapper<SolicitudExpreso> getPlwTrayectosExpresos() {
		return plwTrayectosExpresos;
	}

	public void setPlwTrayectosExpresos(PagedListWrapper<SolicitudExpreso> plwTrayectosExpresos) {
		this.plwTrayectosExpresos = plwTrayectosExpresos;
	}
}
