package com.rjconsultores.ventaboletos.web.gui.controladores.relatorios;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Datebox;
import org.zkoss.zul.Intbox;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.Corrida;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.GrupoRuta;
import com.rjconsultores.ventaboletos.entidad.Ruta;
import com.rjconsultores.ventaboletos.relatorios.impl.RelatorioIndiceIRK;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.service.CorridaService;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.service.GrupoRutaService;
import com.rjconsultores.ventaboletos.service.RutaService;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderCorridaOrigemDestino;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderRelatorioIndiceIRK;

@Controller("relatorioIndiceIRKController")
@Scope("prototype")
public class RelatorioIndiceIRKController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	private static Logger log = LogManager.getLogger(RelatorioIndiceIRKController.class);

	@Autowired
	private DataSource dataSourceRead;

	@Autowired
	private EmpresaService empresaService;

	@Autowired
	private CorridaService corridaService;

	@Autowired
	private RutaService rutaService;

	@Autowired
	private GrupoRutaService grupoRutaService;

	private Datebox datInicial;
	private Datebox datFinal;

	private Combobox cmbEmpresa;
	private List<Empresa> lsEmpresa;

	private Combobox cmbGrupoRuta;
	private List<GrupoRuta> lsGrupoRuta;

	private MyListbox linhaList;
	private MyListbox linhaListSelList;
	private Textbox txtPalavraPesquisaLinha;

	private ArrayList<Ruta> lsNumLinha = new ArrayList<Ruta>();
	private MyListbox servicoList;
	private MyListbox servicoListSelList;
	private Intbox txtCorridaId;

	private ArrayList<Corrida> lsNumServico = new ArrayList<Corrida>();

	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void executarRelatorio() throws Exception {
		try {
			Map<String, Object> parametros = new HashMap<String, Object>();
			// Filtro Data Início
			Timestamp dataInicio = new Timestamp((DateUtil.inicioFecha((java.util.Date) datInicial.getValue()).getTime()));
			parametros.put("DATA_INICIO", dataInicio);
			
			// Filtro Data Final
			Timestamp dataFinal = new Timestamp((DateUtil.fimFecha((java.util.Date) datFinal.getValue()).getTime()));
			parametros.put("DATA_FINAL", dataFinal);
			
			// Filtro usuário Id
			parametros.put("USUARIO_ID", UsuarioLogado.getUsuarioLogado().getUsuarioId().toString());
			
			// Filtro nome usuário
			parametros.put("NOME_USUARIO", UsuarioLogado.getUsuarioLogado().getNombusuario().toString());

			// Filtro Empresa
			Empresa empresa = null;
			if (cmbEmpresa.getSelectedItem() != null) {
				empresa = (Empresa) cmbEmpresa.getSelectedItem().getValue();
			}
			parametros.put("EMPRESA", empresa);
			parametros.put("NOMBEMPRESA", empresa != null ? empresa.getNombempresa() : "TODOS");

			// Filtro Grupo Ruta
			GrupoRuta grupoRuta = null;
			if (cmbGrupoRuta.getSelectedItem() != null) {
				grupoRuta = (GrupoRuta) cmbGrupoRuta.getSelectedItem().getValue();
			}
			parametros.put("GRUPORUTA_ID", grupoRuta != null ? grupoRuta.getGrupoRutaId() : "TODOS");
			parametros.put("DESCGRUPO", grupoRuta != null ? grupoRuta.getDescGrupo() : "TODOS");
			// Filtro Linha
			lsNumLinha = new ArrayList(Arrays.asList(linhaListSelList.getData()));
			String rutasIds = getRutasIds(lsNumLinha);
			parametros.put("LINHA_FILTRO", lsNumLinha != null && !lsNumLinha.isEmpty() ? rutasIds : "TODOS");
			parametros.put("lsNumLinha", lsNumLinha);

			// Filtro serviço
			lsNumServico = new ArrayList(Arrays.asList(servicoListSelList.getData()));
			String corridasIds = getCorridasIds(lsNumServico);
			parametros.put("SERVICO_FILTRO", lsNumServico != null &&  !lsNumServico.isEmpty()? corridasIds : "TODOS");
			parametros.put("lsNumServico", lsNumServico);

			// Instancia o relatório
			Relatorio relatorio = new RelatorioIndiceIRK(parametros, dataSourceRead.getConnection());
			Map<String, Object> args = new HashMap<String, Object>();
			args.put("relatorio", relatorio);

			// Abre a janela do relatório
			openWindow("/component/reportView.zul",
					Labels.getLabel("relatorioIndiceIRKController.window.title"), args, MODAL);
		} catch (Exception e) {
			log.error("Erro ao executar relatório", e);
		}

	}

	private String getCorridasIds(ArrayList<Corrida> lsNumServico) {
		StringBuilder corridasIds = new StringBuilder(); 
		for (int i =0; i< lsNumServico.size() ; i++) {
			corridasIds.append(lsNumServico.get(i).getId().getCorridaId());
			if(lsNumServico.size() > 1 && i < lsNumServico.size() -1) {
				corridasIds.append(", ");
			}
		}
		return corridasIds.toString();
	}

	private String getRutasIds(ArrayList<Ruta> lsNumLinha) {
		StringBuilder rutasIds = new StringBuilder(); 
		for (int i =0; i< lsNumLinha.size() ; i++) {
			rutasIds.append(lsNumLinha.get(i).getRutaId());
			if(lsNumLinha.size() > 1 && i < lsNumLinha.size() -1) {
				rutasIds.append(", ");
			}
		}
		return rutasIds.toString();
	}

	public void onClick$btnExecutarRelatorio(Event ev) throws Exception {
		executarRelatorio();
	}

	public void onClick$btnPesquisaLinha(Event ev) {
		executarPesquisaLinha();
	}

	public void onClick$btnLimparLinha(Event ev) {
		linhaList.clearSelection();
		lsNumLinha.clear();
	}

	public void onDoubleClick$servicoList(Event ev) {
		Corrida corridaAux = (Corrida) servicoList.getSelected();
		servicoListSelList.addItemNovo(corridaAux);
	}

	public void onDoubleClick$linhaList(Event ev) {
		Ruta rutaAux = (Ruta) linhaList.getSelected();
		linhaListSelList.addItemNovo(rutaAux);
	}

	private void executarPesquisaLinha() {

		String palavraPesquisaRuta = txtPalavraPesquisaLinha.getText();
		linhaList.setData(rutaService.buscaRuta(palavraPesquisaRuta));

		if (linhaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("relatorioIndiceIRKController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	private void executarPesquisaServico() {

		Integer corridaId = txtCorridaId.getValue();
		servicoList.setData(corridaService.buscarGroupCorrridaId(corridaId, datInicial.getValue(), datFinal.getValue()));

		if (servicoList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("relatorioLinhasHorarioController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	public void onClick$btnLimparServico(Event ev) {
		servicoList.clearSelection();
		servicoList.setData(new ArrayList<Corrida>());
		lsNumServico.clear();
	}

	public void onClick$btnPesquisaServico(Event ev) {
		executarPesquisaServico();
	}

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);
		lsEmpresa = empresaService.obtenerTodos();

		lsGrupoRuta = grupoRutaService.obtenerTodos();

		linhaList.setItemRenderer(new RenderRelatorioIndiceIRK());
		linhaListSelList.setItemRenderer(new RenderRelatorioIndiceIRK());

		servicoList.setItemRenderer(new RenderCorridaOrigemDestino());
		servicoListSelList.setItemRenderer(new RenderCorridaOrigemDestino());
	}

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

	public List<GrupoRuta> getLsGrupoRuta() {
		return lsGrupoRuta;
	}

	public void setLsGrupoRuta(List<GrupoRuta> lsGrupoRuta) {
		this.lsGrupoRuta = lsGrupoRuta;
	}
}
