package com.rjconsultores.ventaboletos.web.gui.controladores.relatorios;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Datebox;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.relatorios.impl.RelatorioVendasConexao;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEstandar;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxPuntoVenta;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;

@Controller("relatorioVendasConexaoController")
@Scope("prototype")
public class RelatorioVendasConexaoController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	
	@Autowired
	private DataSource dataSourceRead;
	
	private MyComboboxEstandar cmbEmpresa;
	private List<Empresa> lsEmpresa;
	
	private Datebox dataInicial;
	private Datebox dataFinal;
	
	private MyComboboxPuntoVenta cmbPuntoVenta;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);
		this.lsEmpresa = UsuarioLogado.getUsuarioLogado().getEmpresa();
	}

	public void onClick$btnExecutarRelatorio(Event ev) throws Exception {
		excutarRelatorios();
	}

	public void excutarRelatorios() throws SQLException, Exception {

		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
		Date dataDe = dataInicial.getValue();
		Date dataAte = dataFinal.getValue();

		Timestamp fecVentaInicial = new Timestamp(DateUtil.inicioFecha(dataDe).getTime());
		Timestamp fecVentaFinal = new Timestamp(DateUtil.fimFecha(dataAte).getTime());

		Map<String, Object> parametros = new HashMap<String, Object>();
		parametros.put("fecInicio", sdf.format(dataDe));
		parametros.put("fecFinal", sdf.format(dataAte));

		parametros.put("dataFiltroInicial", fecVentaInicial);
		parametros.put("dataFiltroFinal", fecVentaFinal);

		Comboitem itemEmpresa = cmbEmpresa.getSelectedItem();
		if (itemEmpresa != null) {
			Empresa empresa = (Empresa) itemEmpresa.getValue();
			parametros.put("EMPRESA_ID", empresa.getEmpresaId());
			parametros.put("empresa", empresa.getNombempresa());
		} else {
			parametros.put("empresa", "Todas;");
		}
		
		Comboitem itemPuntoventa = cmbPuntoVenta.getSelectedItem();
		if(itemPuntoventa != null) {
			PuntoVenta puntoVenta = (PuntoVenta) itemPuntoventa.getValue();
			if(puntoVenta.getPuntoventaId() > -1) {
				parametros.put("PUNTOVENTA_ID", puntoVenta.getPuntoventaId());
				parametros.put("puntoventa", puntoVenta.getNombpuntoventa());
			} else {
				parametros.put("puntoventa", "Todas;");
			}
		} else {
			parametros.put("puntoventa", "Todas;");
			
		}

		Map<String, Object> args = new HashMap<String, Object>();
		Relatorio relatorio = new RelatorioVendasConexao(parametros, dataSourceRead.getConnection());
		args.put("relatorio", relatorio);

		openWindow("/component/reportView.zul",
				Labels.getLabel("indexController.mniRelatorioVendasConexao.label"), args, MODAL);

	}
	
	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

}
